@@ -35,6 +35,14 @@
 #include "about.h"
 
 
+#if defined(__APPLE__)
+#include "macos.h"
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
+#endif
+
 #ifdef WIN32
 #define LOCALEDIR "locale"
 #endif
@@ -54,6 +62,7 @@
 #endif
 
 #include "tuxpaint-config2.h"
+#include "debug.h"  /* must come after FL includes to avoid printf expansion error */
 
 /* Local function prototypes: */
 
@@ -90,7 +99,7 @@ void setup(int argc, char *argv[])
   int i;
   int show_usage;
   int exit_val;
-  char * tmp_str;
+  char *tmp_str;
 
 #if !defined (__WIN32__) && !defined(__APPLE__) && !defined(__BEOS__)
   const struct paper *ppr;
@@ -187,9 +196,12 @@ void setup(int argc, char *argv[])
 
 
   tmp_str = setlocale(LC_ALL, "");
-  if (tmp_str != NULL) {
+  if (tmp_str != NULL)
+  {
     tpc_locale = strdup(tmp_str);
-  } else {
+  }
+  else
+  {
     tpc_locale = NULL;
   }
 
@@ -232,4 +244,30 @@ void setup(int argc, char *argv[])
   while (ppr != NULL);
 
 #endif
+
+#if defined(FC_DEBUG)
+  /*
+   * Enable fontconfig debugging. See "debug.h"
+   */
+  putenv((char *)("FC_DEBUG=" FC_DEBUG));
+#endif
+
+#if defined(__APPLE__)
+  char cwd[1024];
+
+  /* Redirect output */
+  freopen("/tmp/tuxpaint-config.log", "w", stdout);
+  dup2(fileno(stdout), fileno(stderr));
+  setvbuf(stdout, NULL, _IONBF, 0);
+  setvbuf(stderr, NULL, _IONBF, 0);
+
+  /* Pango uses Fontconfig which requires /opt/local/etc/fonts/fonts.conf. This
+   * file may not exist on the runtime system, however, so we copy the file
+   * into our app bundle at compile time, and tell Fontconfig here to look for
+   * the file within the app bundle. */
+  macos_appdir();
+  getcwd(cwd, sizeof(cwd));
+  DEBUG_PRINTF("cwd: %s\n", cwd);
+  putenv((char *)"FONTCONFIG_PATH=Contents/Resources/etc");
+#endif
 }
