@@ -5,39 +5,45 @@
 
 #include <windows.h>
 #include "win32utils.h"
+#include <stdio.h>
 
 
 #ifdef __CYGWIN__
-  #include <stdio.h>
-  #include <sys/stat.h>
+#include <sys/stat.h>
 
-  void _mkdir(const char* dir)
-  {
-    mkdir(dir, 0777);
-  }
+void _mkdir(const char *dir)
+{
+  mkdir(dir, 0777);
+}
 #else
-  #include <direct.h>
+#include <direct.h>
+#define snprintf      _snprintf
 #endif
 
+extern HINSTANCE fl_display;    // hInstance on win32 platform
 
 /*
   Read access to Windows Registry
 */
-static HRESULT ReadRegistry(const char *key, const char *option, char *value, int size)
+static HRESULT ReadRegistry(const char *key, const char *option, char *value,
+                            int size)
 {
-  LONG	res;
-  HKEY	hKey = NULL;
+  LONG res;
+  HKEY hKey = NULL;
 
   res = RegOpenKeyEx(HKEY_CURRENT_USER, key, 0, KEY_READ, &hKey);
   if (res != ERROR_SUCCESS)
     goto err_exit;
-  res = RegQueryValueEx(hKey, option, NULL, NULL, (LPBYTE)value, (LPDWORD)&size);
+  res =
+    RegQueryValueEx(hKey, option, NULL, NULL, (LPBYTE) value,
+                    (LPDWORD) & size);
   if (res != ERROR_SUCCESS)
     goto err_exit;
   res = ERROR_SUCCESS;
 
 err_exit:
-  if (hKey) RegCloseKey(hKey);
+  if (hKey)
+    RegCloseKey(hKey);
   return HRESULT_FROM_WIN32(res);
 }
 
@@ -51,8 +57,8 @@ static char *remove_slash(char *path)
   if (!len)
     return path;
 
-  if (path[len-1] == '/' || path[len-1] == '\\')
-    path[len-1] = 0;
+  if (path[len - 1] == '/' || path[len - 1] == '\\')
+    path[len - 1] = 0;
 
   return path;
 }
@@ -64,10 +70,11 @@ static char *remove_slash(char *path)
 */
 char *GetDefaultSaveDir(const char *suffix)
 {
-  char          prefix[MAX_PATH];
-  char          path[2*MAX_PATH];
-  const char   *key    = "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders";
-  const char   *option = "AppData";
+  char prefix[MAX_PATH];
+  char path[2 * MAX_PATH];
+  const char *key =
+    "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders";
+  const char *option = "AppData";
   HRESULT hr = S_OK;
 
   if (SUCCEEDED(hr = ReadRegistry(key, option, prefix, sizeof(prefix))))
@@ -80,4 +87,21 @@ char *GetDefaultSaveDir(const char *suffix)
   return strdup("userdata");
 }
 
+/*
+  Returns heap string containing the application executable directory path.
+*/
+char *GetApplicationDir(void)
+{
+  char path[2 * MAX_PATH];
+  char *p;
 
+  if (GetModuleFileName(fl_display, path, sizeof(path)))
+  {
+    if ((p = strrchr(path, '\\')) != NULL)
+    {
+      *p = 0;
+      return strdup(path);
+    }
+  }
+  return strdup("");
+}
